/**
 *  @brief example code for liboauth using http://term.ie/oauth/example
 *  @file oauthexample.c
 *  @author Robin Gareus <robin@gareus.org>
 *
 * Copyright 2008 Robin Gareus <robin@gareus.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

// tweet: written by S.Fuchita (soramimi)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vector>
#include <string>
#include "twicpp/socket.h"
#include "twicpp/oauth.h"

/**
 * split and parse URL parameters replied by the test-server
 * into <em>oauth_token</em> and <em>oauth_token_secret</em>.
 */
bool parse_reply(const char *reply, std::string *token, std::string *secret)
{
	std::vector<std::string> arr;

	char const *end = reply + strlen(reply);
	char const *left = reply;
	char const *right = left;
	while (1) {
		int c = 0;
		if (right < end) {
			c = *right;
		}
		if (c == 0 || c == '&') {
			std::string str(left, right);
			arr.push_back(str);
			if (c == 0) {
				break;
			}
			right++;
			left = right;
		}
		right++;
	}

	char const *oauth_token = 0;
	char const *oauth_token_secret = 0;

	for (std::vector<std::string>::const_iterator it = arr.begin(); it != arr.end(); it++) {
		if (strncmp(it->c_str(), "oauth_token=", 12) == 0) {
			oauth_token = it->c_str() + 12;
		} else if (strncmp(it->c_str(), "oauth_token_secret=", 19) == 0) {
			oauth_token_secret = it->c_str() + 19;
		}
	}

	if (oauth_token && oauth_token_secret) {
		if (token) {
			*token = oauth_token;
		}
		if (secret) {
			*secret = oauth_token_secret;
		}
		return true;
	}

	return false;
}

void tweet(char const *text)
{
	static const char *api_uri = "http://api.twitter.com/1.1/statuses/update.json";
	static const char *request_token_uri = "http://api.twitter.com/oauth/request_token";
	static const char *authorize_uri = "http://api.twitter.com/oauth/authorize";
	static const char *access_token_uri = "http://api.twitter.com/oauth/access_token";

	std::string c_key; //< consumer key
	std::string c_secret; //< consumer secret

	std::string t_key;
	std::string t_secret;

	std::string req_url;
	std::string postarg;
	std::string reply;

#include "keys.txt"

	// call Twitter API

	std::string uri = api_uri;
	uri += "?status=";
	uri += oauth_url_escape(text);

	req_url = oauth_sign_url2(uri.c_str(), &postarg, OA_HMAC, 0, c_key.c_str(), c_secret.c_str(), t_key.c_str(), t_secret.c_str());
	reply = oauth_http_post(req_url.c_str(), postarg.c_str(), false);
}

int main (int argc, char **argv)
{
	if (argc < 2) {
		return 0;
	}
	try {
		Socket::initialize();
		tweet(argv[1]);
	} catch (char const *error) {
		puts(error);
	}
	return 0;
}


